package paraboloide;

import java.awt.Color;

import com.jogamp.opengl.GL2;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.glu.GLU;

public class GLRenderer implements GLEventListener {

	// funzione z=f(x,y)
	private double f(double x, double y) {
		return Math.sin(x)+Math.sin(y);
	}

	private double a = 0.0; // angolo di rotazione
	private double x0=0, y0=-10, z0=20;
	private int width=400, height=300;
	

	@Override
	public void display(GLAutoDrawable drawable) {
		double dy = 0.5;
		double dx = 0.5;
		double z1 = 0;
		double z2 = 0;
		double z3 = 0;
		double z4 = 0;
		GL2 gl = drawable.getGL().getGL2();
		GLU glu = new GLU();

		// cancella l'immagine e il buffer per il test di profondit
		gl.glClear(GL2.GL_COLOR_BUFFER_BIT | GL2.GL_DEPTH_BUFFER_BIT);

		

		// imposta la matrice di trasformazione per la proiezione
		gl.glMatrixMode(GL2.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(45, (float) width / height, 5, 120);
		glu.gluLookAt(x0, y0, z0, 0, 0, 0, 0, 0, 1);

		// imposta la matrice di trasformazione per gli oggetti
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
		//gl.glRotated(a, 0, 0, 1);// rotazione variabile di a intorno a Z

		for (double x = -20; x < 20; x += dx) {
			for (double y = -20; y < 20; y += dy) {
				z1 = f(x, y);
				z2 = f(x + dx, y);
				z3 = f(x + dx, y + dy);
				z4 = f(x, y + dy);

				gl.glBegin(GL2.GL_QUADS);
				gl.glColor3d(R(z1), G(z1), B(z1));
				gl.glVertex3d(x, y, z1); // vertice A
				gl.glColor3d(R(z2), G(z2), B(z2));
				gl.glVertex3d(x + dx, y, z2); // vertice B
				gl.glColor3d(R(z3), G(z3), B(z3));
				gl.glVertex3d(x + dx, y + dy, z3); // vertice C
				gl.glColor3d(R(z4), G(z4), B(z4));
				gl.glVertex3d(x, y + dy, z4); // vertice D
				gl.glEnd();

				gl.glBegin(GL2.GL_LINES);
				gl.glColor3d(1, 1, 1);
				gl.glVertex3d(x, y, z1 - 0.03); // vertice A
				gl.glVertex3d(x + dx, y, z2 - 0.03); // vertice B
				gl.glVertex3d(x + dx, y, z2 - 0.03); // vertice B
				gl.glVertex3d(x + dx, y + dy, z3 - 0.03); // vertice C
				gl.glVertex3d(x + dx, y + dy, z3 - 0.03); // vertice C
				gl.glVertex3d(x, y + dy, z4 - 0.03); // vertice D
				gl.glVertex3d(x, y + dy, z4 - 0.03); // vertice D
				gl.glVertex3d(x, y, z1 - 0.03); // vertice A
				gl.glEnd();

				gl.glBegin(GL2.GL_LINES);
				gl.glColor3d(1, 1, 1);
				gl.glVertex3d(x, y, z1 + 0.03); // vertice A
				gl.glVertex3d(x + dx, y, z2 + 0.03); // vertice B
				gl.glVertex3d(x + dx, y, z2 + 0.03); // vertice B
				gl.glVertex3d(x + dx, y + dy, z3 + 0.03); // vertice C
				gl.glVertex3d(x + dx, y + dy, z3 + 0.03); // vertice C
				gl.glVertex3d(x, y + dy, z4 + 0.03); // vertice D
				gl.glVertex3d(x, y + dy, z4 + 0.03); // vertice D
				gl.glVertex3d(x, y, z1 + 0.03); // vertice A
				gl.glEnd();
			}
		}
		//a += 0.8;
	}

	@Override
	public void dispose(GLAutoDrawable drawable) {
		// TODO Auto-generated method stub

	}

	@Override
	public void init(GLAutoDrawable drawable) {
		GL2 gl = drawable.getGL().getGL2();

		// attiva il test di profondit per disegnare solo le parti visibili
		gl.glEnable(GL2.GL_DEPTH_TEST);
	}

	@Override
	public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
		GL2 gl = drawable.getGL().getGL2();
		GLU glu = new GLU();
		width=w;
		height=h;

		// imposta la matrice di trasformazione per la proiezione
		gl.glMatrixMode(GL2.GL_PROJECTION);
		gl.glLoadIdentity();
		glu.gluPerspective(45, (float) w / h, 5, 50);
		glu.gluLookAt(x0, y0, z0, 0, 0, 0, 0, 0, 1);

		// imposta la matrice di trasformazione per gli oggetti
		gl.glMatrixMode(GL2.GL_MODELVIEW);
		gl.glLoadIdentity();
	}

	private double R(double z) {
		return (Color.getHSBColor((float) z / 6.0f, 1, 1).getRed()) / 255.0;
	}

	private double G(double z) {
		return (Color.getHSBColor((float) z / 6.0f, 1, 1).getGreen()) / 255.0;
	}

	private double B(double z) {
		return (Color.getHSBColor((float) z / 6.0f, 1, 1).getBlue()) / 255.0;
	}
	
	public double getX0() {
		return x0;
	}

	public void setX0(double x0) {
		this.x0 = x0;
	}

	public double getY0() {
		return y0;
	}

	public void setY0(double y0) {
		this.y0 = y0;
	}

	public double getZ0() {
		return z0;
	}

	public void setZ0(double z0) {
		this.z0 = z0;
	}

}
